#ifndef ROOMMODEL_H
#define ROOMMODEL_H

#include <QAbstractListModel>
#include <QSharedPointer>

#include <map>
#include <string>
#include <vector>

#include "models/roommodel.h"

class RoomListModel : public QAbstractListModel {
    Q_OBJECT
  public:
    enum RoomRoles {
        RoomNameRole,
        UnreadNotificationsRole,
        LastMessageRole,
        RoomPictureRole,
        TagRole,
    };

    QHash<int, QByteArray> roleNames() const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;

    QSharedPointer<Room> room(std::string id) { return rooms[id]; }

    explicit RoomListModel(QObject *parent = 0);

  public slots:
    void addRoom(QSharedPointer<Room> room);
    void addInvite(QSharedPointer<Room> room);
    void updateRoom(QSharedPointer<Room> room);

  private:
    QModelIndex calcIndex(std::string id, const std::vector<QSharedPointer<Room>> &invites,
                          const std::vector<QSharedPointer<Room>> &other);
    std::vector<QSharedPointer<Room>> invites;
    std::vector<QSharedPointer<Room>> joinedRooms;
    std::map<std::string, QSharedPointer<Room>> rooms;
};

#endif // ROOMMODEL_H
