#pragma once

#include <string>
#include <vector>

#include <QDateTime>
#include <QString>

#include <mtx/events/collections.hpp>

struct DescInfo {
    QString event_id;
    QString username;
    QString userid;
    QString body;
    QString timestamp;
    QDateTime datetime;
};

struct Room {
    std::string id, lastMessage, tag;

    int unreadNotifications = 0;

    //! The calculated name of the room.
    std::string name;
    //! The topic of the room.
    std::string topic;
    //! The calculated avatar url of the room.
    std::string avatar_url;
    //! Whether or not the room is an invite.
    bool is_invite = false;
    //! Total number of members in the room.
    int16_t member_count = 0;
    //! Who can access to the room.
    mtx::events::state::JoinRule join_rule = mtx::events::state::JoinRule::Public;
    bool guest_access = false;
    //! Metadata describing the last message in the timeline.
    DescInfo msgInfo;
    //! The list of tags associated with this room
    std::vector<std::string> tags;

  private:
    std::vector<mtx::events::collections::TimelineEvent> events;
    std::string prev_batch;
};
