#include "sync.h"

#include <QtDebug>

#include "client.h"
#include "debug_out.h"

void Sync::startSync() {
    if (!isRunning()) {
        start(LowPriority);
    } else {
        stop = false;
        condition.wakeOne();
    }
}

Sync::~Sync() { stopSync(); }

void Sync::stopSync() {
    this->exit = true;
    this->stop = true;
    http::client().shutdown();
    condition.wakeOne();
    wait();
}

void Sync::run() { emit nextSync(); }

void Sync::sync() {
    qDebug() << "sync";

    if (this->stop) {
        if (this->exit) {
            qDebug() << "sync exiting";
            return;
        } else {
            qDebug() << "sync pausing";
            mutex.lock();
            this->condition.wait(&mutex);
            mutex.unlock();
        }
    }

    mtx::http::SyncOpts opts = {};
    opts.since = since;
    http::client().sync(opts, [this](const mtx::responses::Sync &res, mtx::http::RequestErr err) {
        if (err) {
            qDebug() << "Failed sync";
            qDebug() << *err;
            return;
        }

        this->since = res.next_batch;

        for (const std::pair<std::string, mtx::responses::JoinedRoom> &join : res.rooms.join) {
            const mtx::responses::JoinedRoom &r = join.second;

            bool createRoom = false;
            QSharedPointer<Room> room = rooms.room(join.first);
            if (!room) {
                room = QSharedPointer<Room>(new Room);
                createRoom = true;
            }

            room->id = join.first;
            qDebug() << QString::fromStdString(room->id);

            for (const auto &e : r.state.events) {
                // if (const auto n = boost::get<mtx::events::StateEvent<states::Name>>(&e))
                if (const auto n = boost::get<mtx::events::StateEvent<mtx::events::state::Name>>(&e)) {
                    room->name = n->content.name;
                    qDebug() << "Name event: " << QString::fromStdString(n->content.name);
                }
            }
            for (const auto &e : r.timeline.events) {
                if (const auto n = boost::get<mtx::events::StateEvent<mtx::events::state::Name>>(&e)) {
                    room->name = n->content.name;
                    qDebug() << "Name event: " << QString::fromStdString(n->content.name);
                }
            }

            room->unreadNotifications = r.unread_notifications.highlight_count;
            if (createRoom)
                emit newRoom(room);
            else
                emit roomUpdated(room);
        }

        for (const std::pair<std::string, mtx::responses::InvitedRoom> &invite : res.rooms.invite) {
            const mtx::responses::InvitedRoom &r = invite.second;

            bool createRoom = false;
            QSharedPointer<Room> room = rooms.room(invite.first);
            if (!room) {
                room = QSharedPointer<Room>(new Room);
                createRoom = true;
            }

            room->id = invite.first;
            room->tags.push_back("invite");
            room->is_invite = true;
            qDebug() << QString::fromStdString(room->id);

            for (const auto &e : r.invite_state) {
                if (const auto n = boost::get<mtx::events::StrippedEvent<mtx::events::state::Name>>(&e)) {
                    room->name = n->content.name;
                    qDebug() << "Name event: " << QString::fromStdString(n->content.name);
                }
            }

            if (createRoom)
                emit newInvite(room);
            else
                emit roomUpdated(room);
        }

        emit nextSync();
    });
}
