#pragma once

#include <atomic>

#include <QMutex>
#include <QThread>
#include <QWaitCondition>

#include "login.h"
#include "models/roomlistmodel.h"

class Sync : public QThread {
    Q_OBJECT

  public:
    Sync(RoomListModel &rooms, Login *login, QObject *parent = 0)
        : QThread(parent), rooms(rooms), stop(false), exit(false) {
        connect(this, &Sync::nextSync, this, &Sync::sync);
    }
    ~Sync();
    void run();

  public slots:
    void startSync();
    void stopSync();

  private slots:
    void sync();

  signals:
    void newRoom(QSharedPointer<Room> room);
    void newInvite(QSharedPointer<Room> room);
    void roomUpdated(QSharedPointer<Room> room);
    void nextSync();

  private:
    RoomListModel &rooms;
    QWaitCondition condition;
    QMutex mutex;
    std::atomic<bool> stop, exit;
    std::string since;
};
